// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include <getopt.h>
#include <inttypes.h>
#include <math.h>
#include <regex.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <xma.h>

#include "xlnx_app_utils.h"
#include "xlnx_ml_common.h"

typedef struct XlnxMLArguments {
    XmaLogLevelType log_level;      /* -log                    */
    XmaLogType      log_location;   /* -log-location */
    char*           log_file;       /* -log-file */
    uint            device_id;      /* -d                      */
    int             input_width;    /* -w                      */
    int             input_height;   /* -h                      */
    int             fps_num;        /* -fps                    */
    int             fps_den;        /* inferred */
    XmaFormatType   input_pix_fmt;  /* -pix_fmt                */
    char*           input_file;     /* -i                      */
    char*           model_name;     /* -model_name             */
    char*           model_arg;      /* -model_arg              */
    char*           ml_perf;        /* -perf                   */
    char*           ml_tensor_dump; /* -tensor_dump            */
    char*           ml_inf_repeat;  /* -inference_repeat       */
    size_t          num_frames;     /* -frames                 */
    char*           output_file;    /* -o           */
} XlnxMLArguments;

/**
 * xlnx_ml_get_arguments: Parses the commandline arguments into
 * a struct corresponding to ml arguments
 * @param argc: The number of arguments
 * @param argv: The arguments
 * @return An argument struct containing ml arguments.
 */
XlnxMLArguments xlnx_ml_get_arguments(int argc, char* argv[]);
